@@ -2,4 +2,4 @@
 # encoding: UTF-8
 
 #pylint: disable=W0403
-from sqlplus_commando import SqlplusCommando, OracleParser
+from sqlplus_commando import SqlplusCommando, OracleResponseParser
